"use strict";

exports.__esModule = true;
exports.default = void 0;

var _internalAttributes = _interopRequireDefault(require("../processing/dom/internal-attributes"));

var _formDataEntry = _interopRequireDefault(require("./form-data-entry"));

var bufferUtils = _interopRequireWildcard(require("../utils/buffer"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const BOUNDARY_RE = /;\s*boundary=([^;]*)/i;
/*eslint-disable no-unused-vars*/

var ParserState;
/*eslint-enable no-unused-vars*/

(function (ParserState) {
  ParserState[ParserState["inPreamble"] = 0] = "inPreamble";
  ParserState[ParserState["inHeaders"] = 1] = "inHeaders";
  ParserState[ParserState["inBody"] = 2] = "inBody";
  ParserState[ParserState["inEpilogue"] = 3] = "inEpilogue";
})(ParserState || (ParserState = {}));

class FormData {
  constructor() {
    _defineProperty(this, "boundary", null);

    _defineProperty(this, "_boundaryEnd", null);

    _defineProperty(this, "_epilogue", []);

    _defineProperty(this, "_entries", []);

    _defineProperty(this, "_preamble", []);
  }

  _removeEntry(name) {
    this._entries = this._entries.filter(entry => entry.name !== name);
  }

  _injectFileInfo(fileInfo) {
    const entries = this._getEntriesByName(fileInfo.name);

    for (let idx = 0; idx < fileInfo.files.length; idx++) {
      let entry = entries[idx];

      if (!entry) {
        entry = new _formDataEntry.default();

        this._entries.push(entry);
      }

      entry.addFileInfo(fileInfo, idx);
    }
  }

  _isBoundary(line) {
    return this.boundary.equals(line); // eslint-disable-line no-extra-parens
  }

  _isBoundaryEnd(line) {
    return this._boundaryEnd.equals(line); // eslint-disable-line no-extra-parens
  }

  _getEntriesByName(name) {
    return this._entries.reduce((found, entry) => {
      if (entry.name === name) found.push(entry);
      return found;
    }, []);
  }

  expandUploads() {
    const uploadsEntry = this._getEntriesByName(_internalAttributes.default.uploadInfoHiddenInputName)[0];

    if (uploadsEntry) {
      const body = Buffer.concat(uploadsEntry.body).toString();
      const files = JSON.parse(body);

      this._removeEntry(_internalAttributes.default.uploadInfoHiddenInputName);

      files.forEach(fileInfo => this._injectFileInfo(fileInfo));
    }
  }

  parseContentTypeHeader(header) {
    header = String(header);

    if (header.includes('multipart/form-data')) {
      const boundaryMatch = header.match(BOUNDARY_RE);
      const token = boundaryMatch && boundaryMatch[1];

      if (token) {
        this.boundary = Buffer.from('--' + token);
        this._boundaryEnd = Buffer.from('--' + token + '--');
      }
    }
  }

  parseBody(body) {
    let state = ParserState.inPreamble;
    const lines = bufferUtils.createLineIterator(body);
    let currentEntry = null;

    for (var _iterator = lines, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      const line = _ref;

      if (this._isBoundary(line)) {
        if (currentEntry) this._entries.push(currentEntry);
        state = ParserState.inHeaders;
        currentEntry = new _formDataEntry.default();
      } else if (this._isBoundaryEnd(line)) {
        if (currentEntry) this._entries.push(currentEntry);
        state = ParserState.inEpilogue;
      } else if (state === ParserState.inPreamble) bufferUtils.appendLine(this._preamble, line);else if (state === ParserState.inHeaders) {
        if (line.length) currentEntry.setHeader(line.toString()); // eslint-disable-line no-extra-parens
        else state = ParserState.inBody;
      } else if (state === ParserState.inEpilogue) bufferUtils.appendLine(this._epilogue, line);else if (state === ParserState.inBody) bufferUtils.appendLine(currentEntry.body, line); // eslint-disable-line no-extra-parens

    }
  }

  toBuffer() {
    if (!this._boundaryEnd || !this.boundary) return null;
    let chunks = this._preamble;
    if (chunks.length) chunks.push(bufferUtils.CRLF);

    for (var _iterator2 = this._entries, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
      var _ref2;

      if (_isArray2) {
        if (_i2 >= _iterator2.length) break;
        _ref2 = _iterator2[_i2++];
      } else {
        _i2 = _iterator2.next();
        if (_i2.done) break;
        _ref2 = _i2.value;
      }

      const entry = _ref2;
      chunks.push(this.boundary, bufferUtils.CRLF, entry.toBuffer(), bufferUtils.CRLF);
    }

    chunks.push(this._boundaryEnd, bufferUtils.CRLF);
    chunks = chunks.concat(this._epilogue);
    return Buffer.concat(chunks);
  }

}

exports.default = FormData;
module.exports = exports.default;